<?php

namespace Drupal\share_tabs\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;

/**
 * Defines the MyEntity configuration entity.
 *
 * @ConfigEntityType(
 *   id = "share_tab",
 *   label = @Translation("Share Tab"),
 *   handlers = {
 *     "list_builder" = "Drupal\share_tabs\ShareTabListBuilder",
 *     "form" = {
 *       "add" = "Drupal\share_tabs\Form\ShareTabForm",
 *       "edit" = "Drupal\share_tabs\Form\ShareTabForm",
 *       "delete" = "Drupal\share_tabs\Form\ShareTabDeleteForm",
 *       "duplicate" = "Drupal\share_tabs\Form\ShareTabForm"
 *     }
 *   },
 *   config_prefix = "share_tab",
 *   admin_permission = "administer share tab",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   links = {
 *     "add-form" = "/admin/structure/share-tabs/add",
 *     "edit-form" = "/admin/structure/share-tabs/{share_tab}/edit",
 *     "delete-form" = "/admin/structure/share-tabs/{share_tab}/delete",
 *     "duplicate-form" = "/admin/structure/share-tabs/{share_tab}/duplicate",
 *     "collection" = "/admin/structure/share-tabs"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "tabs",
 *     "default_tab",
 *     "share_method",
 *     "empty_fragment",
 *     "query_param_name",
 *   },
 *   cache = {
 *     "tags" = {"share_tab_list", "share_tab:{id}"}
 *   }
 * )
 */
class ShareTab extends ConfigEntityBase {
  const SHARE_METHOD_FRAGMENT = 'fragment';
  const SHARE_METHOD_QUERY = 'query_param';
  /**
   * The unique ID of the entity.
   *
   * @var string
   */
  protected $id;

  /**
   * The label of the entity.
   *
   * @var string
   */
  protected $label;

  /**
   * Tab details
   *
   * @var array
   */
  protected $tabs;

  /**
   * Default tab
   *
   * @var string
   */
  protected $default_tab;

  /**
   * Share method
   *
   * @var string
   */
  protected $share_method;
  
  /**
   * Empty fragment
   *
   * @var boolean
   */
  protected $empty_fragment = FALSE;

  /**
   * Query parameter name
   *
   * @var string
   */
  protected $query_param_name = 'stab';

  public function setLabel($label) {
    $this->label = $label;
  }

  public function addTab() {
    if (!$this->tabs) {
      $this->tabs = [];
    }
    $key = time();
    $this->tabs[$key] = [
      'title' => '',
      'weight' => 0,
      'entity' => [
        'type' => 'node',
        'id' => NULL,
        'view_mode' => NULL,
      ],
      'name' => [
        'autogenerate' => TRUE,
        'custom' => '',
      ],
      'ajax' => FALSE,
    ];

    if (count($this->tabs) == 1) {
      $this->default_tab = $key;
    }
  }

  public function removeTab($key) {
    if (isset($this->tabs[$key])) {
      unset($this->tabs[$key]);
    }
  }

  public function getTabs() {
    if (!$this->tabs) {
      $this->tabs = [];
    }
    uksort($this->tabs, function($a, $b) {
      $this->tabs[$a]['weight'] = $this->tabs[$a]['weight'] ?? 0;
      $this->tabs[$b]['weight'] = $this->tabs[$b]['weight'] ?? 0;
      if ($this->tabs[$a]['weight'] == $this->tabs[$b]['weight'] ) {
        return 0;
      }
      return ($this->tabs[$a]['weight'] < $this->tabs[$b]['weight']) ? -1 : 1;
    });
    return $this->tabs;
  }

  public function getDefaultTab() {
    return $this->default_tab;
  }

  public function getShareMethod() {
    return $this->share_method ?? static::SHARE_METHOD_FRAGMENT;
  }

  public function getEmptyFragment() {
    return $this->empty_fragment;
  }

  public function getQueryParamterName() {
    return $this->query_param_name;
  }

  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);

    foreach ($this->tabs as &$tab) {
      $tab['title'] = trim($tab['title']);
      $tab['name']['custom'] = trim($tab['name']['custom']);
    }
  }

  public function renderTabContent($key) {
    if (!empty($this->tabs[$key])) {
      $tab = $this->tabs[$key];
      $entity_storage = \Drupal::entityTypeManager()->getStorage($tab['entity']['type']);
      $entity = $entity_storage->load($tab['entity']['id']);
      if ($entity->access('view')) {

        $view_mode = !empty($tab['entity']['view_mode']) ? $tab['entity']['view_mode'] : 'full';
        return \Drupal::entityTypeManager()
          ->getViewBuilder($tab['entity']['type'])
          ->view($entity, $view_mode);
      }
    }
  }
}
